/*
	ProWesS, PROGS Windowing System

	ProWesS main routine
	actions - load configuration data
			- initialise ProWesS thing
			- load objects

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera
	started 7 January 1995
*/

#include <sms.h>
#include <job.h>
#include <win.h>
#include <module.h>

#include "ProWesS.h"
#include "PWTypeDef.h"
#include "PWConfig.h"

#define catch(x) do { if (err=(x)) return err; } while (0)

Error PWThingInit();
void PWCalculateFactors(short xsiz, short ysiz);

static WindowDescriptor report_window={ 350,50,85,80,6,6,0,1,2,7 };


/* here come the global variables */
Job ProWesS;				/* job id for myself */
short InitIsDone=FALSE; 	/* TRUE when initialisation finished */

extern Gstate system_gstate;

static void report(char *format, char *param)
{
	char string[200];
	Window win;

	SMSBeep(2000,20);
	WINOpenFull(&report_window,&win);
	STRFormatPutSafe(string,200,format,param);
	WINPutString(win,string);
	WINPutString(win,"press a key");
	WINGet(win,string);
	JOBZap(MYSELF,ERR_ITNF);
}

Bbox emptyBbox = { 0,0,0,0 };

Error init(char *param)
{
	Error err;
	char line[100]; 			/* place to read line of config file */
	Channel chid;				/* channel id for config file */
	char **params, *path=NULL;
	int i;

	JOBMe(&ProWesS);			/* fill in my job id */

	err=PWThingInit();			/* link in ProWesS DLL linkage */
	if (err) return err;

	LinkPROforma(); 			/* make sure we have a proper PROforma link */

	err=PFGstateInit(-1,NULL,&emptyBbox,&system_gstate,NULL);
	if (err) report("can't initialise system gstate (%d)",(char *)err);

	/* calculate transformation factors for point to pixel and */
	/* pixel to point conversions (both expressed as 16.16 fixpoint */
	/* we need the screen size for this */
	{
		Window win; WindowInfo limits;

		err=WINOpen(2,1,0,0,&win);
		if (err) report("problem : can't determine screen size",NULL);
		err=WINLimits(win,&limits);
		if (err) report("problem : can't determine screen size",NULL);
		WINClose(win);
		PWCalculateFactors(limits.xsiz,limits.ysiz);
	}

	/* handle the configuration file */

	/* find default path for ProWesS_cfg file */
	catch( STRBreak(param,&params) );
	for (i=0 ; params[i] ; i++)
	{
		if (STRSameCI("-path",params[i]))
		{
			if (params[i+1])
				path=params[++i];
			else
				return ERR_IPAR;
		}
		else
			path=params[i];
	}

	/* try to open config file */
	err=IOOpenPath("ProWesS_cfg",OPEN_READ,path,&chid);
	if (err)
		err=IOOpenPath("ProWesS_cfg",OPEN_READ,PATH_PROGDATA,&chid);
	if (err)
		report("can't open \"ProWesS_cfg\", the config file",NULL);
	MEMRelease(params);

	while (!IOTest(chid))
	{
		IOGetString(chid, line, 100);
		err=PWConfig(line);
		switch (err)
		{
		case ERR_OK:
			break;
		case ERR_ITNF:
			report("item or file NOT FOUND while processing\n%s",line);
		case ERR_ISYN:
			report("FILE IS NOT AN EXTERNAL TYPE while processing\n%s",line);
		case ERR_IMEM:
			report("INSUFFICIENT MEMORY for processing\n%s",line);
		case ERR_IPAR:
			report("don't know how to process\n%s",line);
		default:
			report("encountered an UNKNOWN ERROR while processing\n%s",line);
		}
	}
	IOClose(chid);

	InitIsDone=TRUE;

	/* now suspend this job indefinately */
	while (1) JOBSuspend(MYSELF,FOREVER,NULL);

	return ERR_OK;
}
