#ifndef _PW_PWSYSTEM_H

/*
	ProWesS, PROGS Windowing System

	Definition of the internal system, private definitions

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera
	16.04.1997 started
*/

#define _PW_PRIVATE

#ifndef _PW_PROWESS_H
#include "ProWesS.h"
#endif
#ifndef _PW_PWOBJECT_H
#include "PWObject.h"
#endif
#ifndef _PW_PWSYSTEM_H
#include "PWSystem.h"
#endif

/* referenced routines */

Error PWDetermineSize(PWSystem *system);
Error PWRemoveIndicated(PWSystem *system);
Error PWHandleRegion(PWSystem *system, PWRegion *region);
Error PWRedraw(PWSystem *system);
void PWDeterminePosition(PWSystem *system);
Error PWWindowFit(PWSystem *system, pt xinc, pt yinc);
Error PWSetWindow(PWSystem *system);
Error PWSetSprites(PWSystem *system);
Error PWDrawBorders(PWSystem *system);
Error PWSleep(PWSystem *system);

#define PW_CORNER_TOPLEFT	0
#define PW_CORNER_TOPRIGHT	2
#define PW_CORNER_BOTLEFT	4
#define PW_CORNER_BOTRIGHT	6
Error PWWindowMove(PWSystem *system,short move);
Error PWWindowScale(PWSystem *system,short scale);
Error PWScrollScreen(PWSystem *system, int x, int y, char move);
Error PWScrollTouch(PWSystem *system);
Error SoftwareEvent(PWSystem *system);

Error PWClose(PWSystem *system);


/* System data structure */

struct _PWSystem {
	PWPublicSystem; 				/* public portion of ProWesS system */

	struct _PWSystem *next; 		/* next PWSystem for this job */

	PWRegion *regions;				/* tree of info regions in window */
	PWKeyPress *keypresses; 		/* list of keypresses to scan */
									/* data for WINPointerRead */
	PointerInfo ptr_info;			/* the current pointer record */

	void *global;					/* user fillable data - global structure */
	PWDummyTypeState *state;		/* state information for types used */
									/* information about the window/outline */
	pix min_xsiz, min_ysiz; 		/* minimum window size */
	pix max_xsiz, max_ysiz; 		/* maximum window size */
	pix curr_xsiz, curr_ysiz;		/* current window size */

	PWRegion *sleep;				/* sleep region */
	PWRegion *active;				/* currently active region */

	Priority priority;				/* previous priority for SSTATUS_BUTFRAME */

	SubWindowList subwindows;		/* list of subwindows */
	SubWindow *subwinlist[9];		/* array with pointers to subwins */
	SubWindow subwin[8];			/* the definition of the subwindows */

	Error (*ActionInit)(PWObject object);
	Error (*EventLevel1)(PWObject object);
	Error (*EventLevel2)(PWObject object);
	Error (*EventLevel3)(PWObject object);
	Error (*EventLevel4)(PWObject object);
	Error (*EventLevel5)(PWObject object);
	Error (*EventLevel6)(PWObject object);
	Error (*EventLevel7)(PWObject object);
	Error (*EventLevel8)(PWObject object);

	short oldptrx, oldptry; 		/* pointer pos before displaying window */
	pix borderx, bordery;			/* border width for window */
	pix win_ptrx, win_ptry; 		/* preferred pointer position in window */
	PWObject autosize;				/* autoscale object for system */
};

#define SSTATUS_BREAKDOWN	(unsigned long)0x00000002  /* set when activation should be ended */
#define SSTATUS_SLEEP		(unsigned long)0x00000008  /* set when window should be put asleep */
#define SSTATUS_ALLDRAW 	(unsigned long)0x00000010  /* set when all of window needs redraw */
#define SSTATUS_REMOVE		(unsigned long)0x00000020  /* set when an object has to be removed */

#define SSTATUS_NOXSCALE	(unsigned long)0x00000100  /* no horizontal scaling */
#define SSTATUS_NOYSCALE	(unsigned long)0x00000200  /* no vertical scaling */

#define SSTATUS_NOXBORDER	(unsigned long)0x00010000  /* no scaleborder - horizontal */
#define SSTATUS_NOYBORDER	(unsigned long)0x00020000  /* no scaleborder - vertical */
#define SSTATUS_XSCROLLABLE (unsigned long)0x00040000  /* window is larger than screen - x */
#define SSTATUS_YSCROLLABLE (unsigned long)0x00080000  /* window is larger than screen - y */
#define SSTATUS_BUTFRAME	(unsigned long)0x00100000  /* put window in button frame */

#endif _PW_PWSYSTEM_H
