/*
	ProWesS, PROGS Windowing System

	test program - file select

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera
	started 15 August 1995
*/

#include "dir_h"
#include "dev_h"

#include "ProWesS_h"

typedef struct {
	PWObject dir,name;
	PWObject sub,files;
	} Globals;

/* forward declaration */
Error getdir(PWObject object);
Error dirup(PWObject object);
Error devsel(PWObject object);
Error dirdown(PWObject object, char *item);
Error dirsel(PWObject object, char *item);

void init()
{
	Globals g;
	PWObject obj,row,pref;
	PWObject sep1,sep2,sep3,sep4;
	PWObject dirbox, namebox;
	char currdir[IO_MAXDIRECTORY];

	/* get current data directory to fill in directory name */
	DEVDataGet(currdir);

	/* create main window */
	PWCreate(NULL,&obj,PW_TYPE_OUTLINE,
				PW_OUTLINE_SLEEP,
				PW_OUTLINE_QUIT,
				PW_OUTLINE_ACTION_WAKE, &getdir,
				PW_WINDOW_YSIZE, short2pt(540),
				PW_GLOBAL_AUXILIARY, &g,
				NULL);
	PWCreate(obj,&row,PW_TYPE_DIRECTION,
				PW_SCALE_FACTOR, 0,
				NULL);

	/* create edline for directory, with label */
	PWCreate(obj,&dirbox,PW_TYPE_DIRECTION,
/*				  PW_POSITION_BELOW, row, */
				PW_SCALE_FACTOR, 0,
				NULL);
	PWCreate(dirbox,NULL,PW_TYPE_LABEL,
				PW_LABEL_TEXT, "directory",
				PW_SCALE_FACTOR, 0,
				NULL);
	PWCreate(dirbox,&g.dir,PW_TYPE_EDLINE,
				PW_EDLINE_SET, currdir,
				PW_EDLINE_MAXLENGTH, IO_MAXDIRECTORY,
				PW_EDLINE_KEYPRESS, 'd',
				PW_EDLINE_ACTION_AFTER, &getdir,
				NULL);
	PWCreate(dirbox,NULL,PW_TYPE_LOOSE_ITEM,
				PW_LOOSE_CHANGE_STATUS,FALSE,
				PW_LOOSE_ACTION_HIT, &dirup,
				PW_LOOSE_TEXT, "<-",
				PW_LOOSE_KEYPRESS, '<',
				PW_SCALE_FACTOR, 0,
				NULL);

	/* create menus for preferred and subdirectories */
	PWCreate(obj,&sep1,PW_TYPE_SEPARATOR,
/*				  PW_POSITION_BELOW, dirbox, */
				NULL);
	PWCreate(obj,&pref,PW_TYPE_MENU,
				PW_MENU_ADD, "win1_pw_obj",
				PW_MENU_ADD, "win1_pw_test",
				PW_MENU_ADD, "win1_pf_src",
				PW_MENU_ADD, "win1_pf_src_drv",
				PW_MENU_ACTION_SELECT, &dirsel,
				NULL);
	PWCreate(obj,&sep2,PW_TYPE_SEPARATOR,
/*				  PW_POSITION_BELOW, pref, */
				NULL);
	PWCreate(obj,&g.sub,PW_TYPE_MENU,
				PW_MENU_ACTION_SELECT, &dirdown,
				NULL);
	PWCreate(obj,&sep3,PW_TYPE_SEPARATOR,
/*				  PW_POSITION_BELOW, g.sub, */
				NULL);

	/* create edline for filename, with label */
	PWCreate(obj,&namebox,PW_TYPE_DIRECTION,
				PW_SCALE_FACTOR, 0,
				NULL);
	PWCreate(namebox,NULL,PW_TYPE_LABEL,
				PW_LABEL_TEXT, "filename",
				PW_SCALE_FACTOR, 0,
				NULL);
	PWCreate(namebox,&g.name,PW_TYPE_EDLINE,
				PW_EDLINE_MAXLENGTH, IO_MAXFULLNAME,
				PW_EDLINE_KEYPRESS, 'f',
				NULL);

	/* create menu with possible filenames */
	PWCreate(obj,&sep4,PW_TYPE_SEPARATOR,
/*				  PW_POSITION_BELOW, namebox, */
				NULL);
	PWCreate(obj,&g.files,PW_TYPE_MENU,
				PW_SCALE_FACTOR, 3,
				NULL);

	/* create list of devices */
	PWCreate(row,NULL,PW_TYPE_LOOSE_ITEM,
				PW_LOOSE_TEXT, "win1_",
				PW_LOOSE_CHANGE_STATUS, FALSE,
				PW_LOOSE_ACTION_HIT, &devsel,
				NULL);
	PWCreate(row,NULL,PW_TYPE_LOOSE_ITEM,
				PW_LOOSE_TEXT, "win2_",
				PW_LOOSE_CHANGE_STATUS, FALSE,
				PW_LOOSE_ACTION_HIT, &devsel,
				NULL);
	PWCreate(row,NULL,PW_TYPE_LOOSE_ITEM,
				PW_LOOSE_TEXT, "flp1_",
				PW_LOOSE_CHANGE_STATUS, FALSE,
				PW_LOOSE_ACTION_HIT, &devsel,
				NULL);
	PWCreate(row,NULL,PW_TYPE_LOOSE_ITEM,
				PW_LOOSE_TEXT, "ram1_",
				PW_LOOSE_CHANGE_STATUS, FALSE,
				PW_LOOSE_ACTION_HIT, &devsel,
				NULL);

	getdir(obj);

	/* activate */
	PWActivate(obj);
}

Error getdir(PWObject object)
{
	Globals *g;
	DirIterator it;
	char currdir[IO_MAXDIRECTORY];
	FileInfo fi;

	PWQuery(object,PW_GLOBAL_AUXILIARY,&g);
	PWChange(g->dir,PW_EDLINE_GET,IO_MAXDIRECTORY,currdir,NULL);

	/* get subdirectories */
	PWChange(g->sub,PW_MENU_CLEAR,NULL);
	if (!DIRSubOpen(currdir,FALSE,&it))
	{
		while(!DIRSubNext(it,&fi))
			PWChange(g->sub,PW_MENU_ADD_COPY,fi.name,NULL);
		DIRSubClose(it);
	}

	/* get files in directory */
	PWChange(g->files,PW_MENU_CLEAR,NULL);
	if (!DIROpen(currdir,FALSE,&it,NULL))
	{
		while(!DIRNext(it,&fi))
			PWChange(g->files,PW_MENU_ADD_COPY,fi.name,NULL);
		DIRClose(it);
	}

	return ERR_OK;
}

Error dirup(PWObject object)
{
	Globals *g;
	char currdir[IO_MAXDIRECTORY];
	int len;

	PWQuery(object,PW_GLOBAL_AUXILIARY,&g);
	PWChange(g->dir,PW_EDLINE_GET,IO_MAXDIRECTORY,currdir,NULL);

	len=strlen(currdir);
	for (len=strlen(currdir)-1 ; len>0 && currdir[len-1]!='_' ; len--);
	if (len>0)
	{
		currdir[len]='\0';
		PWChange(g->dir,PW_EDLINE_SET,currdir,NULL);
		getdir(object);
	}

	return ERR_OK;
}

Error dirsel(PWObject object, char *item)
{
	Globals *g;
	int len;

	PWQuery(object,PW_GLOBAL_AUXILIARY,&g);

	PWChange(g->dir,PW_EDLINE_SET,item,NULL);
	getdir(object);

	return ERR_OK;
}

Error dirdown(PWObject object, char *item)
{
	Globals *g;
	char currdir[IO_MAXDIRECTORY];

	PWQuery(object,PW_GLOBAL_AUXILIARY,&g);
	PWChange(g->dir,PW_EDLINE_GET,IO_MAXDIRECTORY,currdir,NULL);

	if (currdir[strlen(currdir)-1]!='_')
		STRAppendSafe("_",currdir,IO_MAXDIRECTORY);
	STRAppendSafe(item,currdir,IO_MAXDIRECTORY);
	PWChange(g->dir,PW_EDLINE_SET,currdir,NULL);
	getdir(object);

	return ERR_OK;
}

Error devsel(PWObject object)
{
	Globals *g;
	char olddir[IO_MAXDIRECTORY];
	char newdir[IO_MAXDIRECTORY];
	char *item, *pos;

	/* the item has to contain the underscore in the devicename !! */

	PWQuery(object,PW_GLOBAL_AUXILIARY,&g);
	PWChange(g->dir,PW_EDLINE_GET,IO_MAXDIRECTORY,olddir,NULL);
	PWQuery(object,PW_LOOSE_TEXT,&item);

	if (MEMSame(strlen(item),item,olddir))
		return ERR_OK;

	for (pos=olddir ; *pos && *pos++!='_' ; );
	STRCopySafe(item,newdir,IO_MAXDIRECTORY);
	STRAppendSafe(pos,newdir,IO_MAXDIRECTORY);
	PWChange(g->dir,PW_EDLINE_SET,newdir,NULL);
	getdir(object);

	return ERR_OK;
}
